/* ***************************************************************************
   *                                                                         *
   *   Kindermann                        Wahlpflichtkurs Produktentwicklung  *
   *                                     Schuljahr: 2014-15                  *
   *                                                                         *
   *                             AVR-Flashtool 1.6                           *
   *                                                                         *
   *                         (Crossplattform-Version)                        *
   *                                                                         *
   ***************************************************************************/


#include "main.h"
#include "aboutapp.h"
#include "communicate.h"
#include <wx/wxprec.h>
#include <wx/wx.h>
#include <wx/hyperlink.h>
#include "logo.xpm"


BEGIN_EVENT_TABLE (AppAbout, wxDialog)
    EVT_TIMER (myID_ABOUTTIMER, AppAbout::OnTimerEvent)
END_EVENT_TABLE ()


AppAbout::AppAbout (wxWindow *parent,
                    int milliseconds)
        : wxDialog (parent, -1, _("AVR-Flashtool"),
                    wxDefaultPosition, wxDefaultSize,
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER) {

    m_timer = NULL;
    if (milliseconds > 0) {
        m_timer = new wxTimer (this, myID_ABOUTTIMER);
        m_timer->Start (milliseconds, wxTIMER_ONE_SHOT);
    }


    SetIcon(wxICON(icon));
    wxBoxSizer *totalpane = new wxBoxSizer (wxVERTICAL);
    wxStaticText *appname = new wxStaticText(this, -1, APP_NAME);
    appname->SetFont (wxFont (20, wxDEFAULT, wxNORMAL, wxBOLD));
    totalpane->Add (appname, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 30);

    wxStaticText *appversion = new wxStaticText(this, -1, APP_VERSION);
    appversion->SetFont (wxFont (12, wxDEFAULT, wxNORMAL, wxBOLD));
    totalpane->Add (appversion, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 30);
    totalpane->Add (0, 20);

    wxBoxSizer *aboutpane = new wxBoxSizer (wxHORIZONTAL);
    wxBitmap bitmap = wxBitmap(wxBitmap(logo_xpm));
    aboutpane->Add (new wxStaticBitmap (this, -1, bitmap),
                    1, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxLEFT | wxRIGHT, 20);


    totalpane->Add (aboutpane, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10);
    totalpane->Add (0, 10);

    wxStaticText *appcopyright = new wxStaticText(this, -1, APP_COPYRIGHT);
    appcopyright->SetFont (wxFont (12, wxDEFAULT, wxNORMAL, wxBOLD));
    totalpane->Add (appcopyright, 0, wxALIGN_CENTER | wxLEFT | wxRIGHT, 30);

    wxHyperlinkCtrl *website = new wxHyperlinkCtrl(this,
                                               wxID_ANY,
                                               wxT("www.dietrich-kindermann.de"),
                                               wxT("www.dietrich-kindermann.de/index.html"));

    totalpane->Add( website, 1, wxALIGN_CENTER | wxALL, 5 );

    wxButton *okButton = new wxButton (this, wxID_OK, _("OK"));
    okButton->SetDefault();
    totalpane->Add (okButton, 0, wxALIGN_CENTER | wxALL, 10);

    SetSizerAndFit (totalpane);

    int width = this->GetSize().GetWidth();
    int height = this->GetSize().GetHeight();
    this->SetMinSize(wxSize(width, height));
    this->SetMaxSize(wxSize(width, height));

    CentreOnParent();
    ShowModal();
}


AppAbout::~AppAbout () {
    if (m_timer)  {
        delete m_timer;
        m_timer = NULL;
    }
}


void AppAbout::OnTimerEvent (wxTimerEvent &WXUNUSED(event)) {
    if (m_timer) delete m_timer;
    m_timer = NULL;
    EndModal (wxID_OK);
}

