/* ***************************************************************************
   *                                                                         *
   *   Kindermann                        Wahlpflichtkurs Produktentwicklung  *
   *                                     Schuljahr: 2014-15                  *
   *                                                                         *
   *                             AVR-Flashtool 1.6                           *
   *                                                                         *
   *                         (Crossplattform-Version)                        *
   *                                                                         *
   ***************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>


#include "panels.h"
#include "helptext.h"
#include "communicate.h"
#include "cbootloader.h"
#include "cflashmem.h"


/******************************************************************************
* Global definitions
******************************************************************************/
  unsigned char dataBuffer[65536 + 256];    /* buffer for file data */

  //const static int progress_step = 0; // current step
  unsigned int progress_step; // current step

  //const static int progress_total_steps = 0; // total steps for upload
  unsigned int progress_total_steps; // total steps for upload

  char* progress_friendly_name; // name of progress section

  //const static int dump_progress = 0; // output computer friendly progress info
  unsigned int dump_progress; // output computer friendly progress info

  //const static int use_ansi = 0; // output ansi control character stuff
  unsigned int use_ansi; // output ansi control character stuff

  //const static int erase_only = 0; // only erase, dont't write file
  unsigned int erase_only; // only erase, dont't write file

  //const static int timeout = 0;
  unsigned int timeout;

  typedef void (*micronucleus_callback)(float progress, Communicate* comm);

  wxString content;

/******************************************************************************/


#ifdef __WINDOWS__
RightPanel::RightPanel(wxPanel * parent)
       : wxPanel(parent, wxID_ANY, wxDefaultPosition,
         wxSize(520, 430), 0)
#endif

#ifdef __LINUX__
RightPanel::RightPanel(wxPanel * parent)
       : wxPanel(parent, wxID_ANY, wxDefaultPosition,
         wxSize(620, 430), 0)
#endif


{
  #ifdef __WINDOWS__
  this->SetSizeHints(wxSize(520, 430), wxSize(520, 430));
  #endif

  #ifdef __LINUX__
  this->SetSizeHints(wxSize(620, 430), wxSize(600, 430));
  #endif

    wxArrayString includeList;
    includeList.Add(wxT("0"));
    includeList.Add(wxT("1"));
    includeList.Add(wxT("2"));
    includeList.Add(wxT("3"));
    includeList.Add(wxT("4"));
    includeList.Add(wxT("5"));
    includeList.Add(wxT("6"));
    includeList.Add(wxT("7"));
    includeList.Add(wxT("8"));
    includeList.Add(wxT("9"));
    includeList.Add(wxT("a"));
    includeList.Add(wxT("b"));
    includeList.Add(wxT("c"));
    includeList.Add(wxT("d"));
    includeList.Add(wxT("e"));
    includeList.Add(wxT("f"));
    includeList.Add(wxT("A"));
    includeList.Add(wxT("B"));
    includeList.Add(wxT("C"));
    includeList.Add(wxT("D"));
    includeList.Add(wxT("E"));
    includeList.Add(wxT("F"));

    wxTextValidator textValidator(wxFILTER_INCLUDE_CHAR_LIST);
    textValidator.SetIncludes(includeList);

	wxFlexGridSizer* fgSizer1;
	fgSizer1 = new wxFlexGridSizer( 4, 1, 0, 0 );
	fgSizer1->SetFlexibleDirection( wxBOTH );
	fgSizer1->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );

	#ifdef __WINDOWS__
	fgSizer1->SetMinSize( wxSize( 500,-1 ) );
	#endif

	#ifdef __LINUX__
	fgSizer1->SetMinSize( wxSize( 610,-1 ) );
	#endif

	wxBoxSizer* bSizer1;
	bSizer1 = new wxBoxSizer( wxHORIZONTAL );

    #ifdef __WINDOWS__
	bSizer1->SetMinSize( wxSize( 500,150 ) );
	#endif

    #ifdef __LINUX__
	bSizer1->SetMinSize( wxSize( 610,150 ) );
	#endif

	wxBoxSizer* bSizer2;
	bSizer2 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer3;
	bSizer3 = new wxBoxSizer( wxVERTICAL );

	wxBoxSizer* bSizer4;
	bSizer4 = new wxBoxSizer( wxHORIZONTAL );

    #ifdef __WINDOWS__
	m_staticText1 = new wxStaticText( this, wxID_ANY, wxT("Vendor ID:     0x"), wxDefaultPosition, wxDefaultSize, 0 );
    #endif

    #ifdef __LINUX__
	m_staticText1 = new wxStaticText( this, wxID_ANY, wxT("Vendor ID:    0x"), wxDefaultPosition, wxDefaultSize, 0 );
    #endif

	m_staticText1->Wrap( -1 );

	#ifdef __WINDOWS__
	m_staticText1->SetMinSize( wxSize( 80,-1 ) );
	m_staticText1->SetMaxSize( wxSize( 80,-1 ) );
	#endif

	#ifdef __LINUX__
	m_staticText1->SetMinSize( wxSize( 110,-1 ) );
	m_staticText1->SetMaxSize( wxSize( 110,-1 ) );
	#endif

	bSizer4->Add( m_staticText1, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

    m_textCtrl1 = new wxTextCtrl( this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, textValidator );

    m_textCtrl1->SetMaxLength(4);
	m_textCtrl1->SetMinSize( wxSize( 50,-1 ) );
	m_textCtrl1->SetMaxSize( wxSize( 50,-1 ) );

	bSizer4->Add( m_textCtrl1, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer3->Add( bSizer4, 1, wxEXPAND, 5 );

	wxBoxSizer* bSizer5;
	bSizer5 = new wxBoxSizer( wxHORIZONTAL );

    #ifdef __WINDOWS__
	m_staticText2 = new wxStaticText( this, wxID_ANY, wxT("Produkt ID:    0x"), wxDefaultPosition, wxDefaultSize, 0 );
    #endif

    #ifdef __LINUX__
	m_staticText2 = new wxStaticText( this, wxID_ANY, wxT("Produkt ID:   0x"), wxDefaultPosition, wxDefaultSize, 0 );
    #endif

	m_staticText2->Wrap( -1 );

	#ifdef __WINDOWS__
	m_staticText2->SetMinSize( wxSize( 80,-1 ) );
	m_staticText2->SetMaxSize( wxSize( 80,-1 ) );
	#endif

	#ifdef __LINUX__
	m_staticText2->SetMinSize( wxSize( 110,-1 ) );
	m_staticText2->SetMaxSize( wxSize( 110,-1 ) );
	#endif

	bSizer5->Add( m_staticText2, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

    m_textCtrl2 = new wxTextCtrl( this, wxID_ANY, wxT(""), wxDefaultPosition, wxDefaultSize, wxALIGN_LEFT, textValidator );

    m_textCtrl2->SetMaxLength(3);
    m_textCtrl2->SetMinSize( wxSize( 50,-1 ) );
	m_textCtrl2->SetMaxSize( wxSize( 50,-1 ) );

	bSizer5->Add( m_textCtrl2, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer3->Add( bSizer5, 1, wxEXPAND, 5 );

	m_button1 = new wxButton( this, ID_FIND, wxT("Gert suchen"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer3->Add( m_button1, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	m_staticText3 = new wxStaticText( this, wxID_ANY, wxT("                     "), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText3->Wrap( -1 );
	bSizer3->Add( m_staticText3, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	bSizer2->Add( bSizer3, 1, wxEXPAND, 5 );

	bSizer1->Add( bSizer2, 1, wxEXPAND, 5 );

	wxBoxSizer* bSizer6;
	bSizer6 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer7;
	bSizer7 = new wxBoxSizer( wxVERTICAL );

	wxBoxSizer* bSizer8;
	bSizer8 = new wxBoxSizer( wxHORIZONTAL );

	m_staticText5 = new wxStaticText( this, wxID_ANY, wxT("Seitengre:"), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText5->Wrap( -1 );

	#ifdef __WINDOWS__
	m_staticText5->SetMinSize( wxSize( 80,-1 ) );
	m_staticText5->SetMaxSize( wxSize( 80,-1 ) );
	#endif

	#ifdef __LINUX__
	m_staticText5->SetMinSize( wxSize( 120,-1 ) );
	m_staticText5->SetMaxSize( wxSize( 120,-1 ) );
	#endif

	bSizer8->Add( m_staticText5, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	m_staticText6 = new wxStaticText( this, wxID_ANY, wxT("00000000"), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText6->Wrap( -1 );
	bSizer8->Add( m_staticText6, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer7->Add( bSizer8, 1, wxEXPAND, 5 );

	wxBoxSizer* bSizer9;
	bSizer9 = new wxBoxSizer( wxHORIZONTAL );

	m_staticText7 = new wxStaticText( this, wxID_ANY, wxT("Programmgre:"), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText7->Wrap( -1 );

	#ifdef __WINDOWS__
	m_staticText7->SetMinSize( wxSize( 80,-1 ) );
	m_staticText7->SetMaxSize( wxSize( 80,-1 ) );
	#endif

	#ifdef __LINUX__
	m_staticText7->SetMinSize( wxSize( 120,-1 ) );
	m_staticText7->SetMaxSize( wxSize( 120,-1 ) );
	#endif

	bSizer9->Add( m_staticText7, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	m_staticText8 = new wxStaticText( this, wxID_ANY, wxT("00000000"), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText8->Wrap( -1 );
	bSizer9->Add( m_staticText8, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer7->Add( bSizer9, 1, wxEXPAND, 5 );

	m_button2 = new wxButton( this, ID_OPEN, wxT("Hex-Datei ffnen"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer7->Add( m_button2, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	m_staticText9 = new wxStaticText( this, wxID_ANY, wxT("                    "), wxDefaultPosition, wxDefaultSize, 0 );
	m_staticText9->Wrap( -1 );
	bSizer7->Add( m_staticText9, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	bSizer6->Add( bSizer7, 1, wxEXPAND, 5 );

	bSizer1->Add( bSizer6, 1, wxEXPAND, 5 );

	wxBoxSizer* bSizer10;
	bSizer10 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer11;
	bSizer11 = new wxBoxSizer( wxVERTICAL );

	wxBoxSizer* bSizer12;
	bSizer12 = new wxBoxSizer( wxVERTICAL );

	wxBoxSizer* bSizer13;
	bSizer13 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer14;
	bSizer14 = new wxBoxSizer( wxVERTICAL );

	m_button3 = new wxButton( this, ID_FLASH, wxT("Programm bertragen"), wxDefaultPosition, wxDefaultSize, wxNO_BORDER );
	bSizer14->Add( m_button3, 0, wxALL|wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL, 5 );

	bSizer13->Add( bSizer14, 1, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer12->Add( bSizer13, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 5 );

	bSizer11->Add( bSizer12, 1, wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 5 );

	wxBoxSizer* bSizer15;
	bSizer15 = new wxBoxSizer( wxHORIZONTAL );

	wxBoxSizer* bSizer16;
	bSizer16 = new wxBoxSizer( wxVERTICAL );

	m_checkBox1 = new wxCheckBox( this, wxID_ANY, wxT("Reboot AVR!"), wxDefaultPosition, wxDefaultSize, 0 );
	m_checkBox1->SetValue(true);
	bSizer16->Add( m_checkBox1, 1, wxALL|wxALIGN_CENTER_VERTICAL|wxALIGN_CENTER_HORIZONTAL, 5 );

	bSizer15->Add( bSizer16, 1, wxEXPAND|wxALIGN_CENTER_VERTICAL, 5 );

	bSizer11->Add( bSizer15, 1, wxEXPAND, 5 );

	m_button4 = new wxButton( this, ID_HELP, wxT("Hilfe"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer11->Add( m_button4, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	wxBoxSizer* bSizer18;
	bSizer18 = new wxBoxSizer( wxHORIZONTAL );

    m_radioBtn1 = new wxRadioButton( this, wxID_ANY, wxT("USB-Board"), wxDefaultPosition, wxDefaultSize, 0 );
	m_radioBtn1->SetValue( true );
	bSizer18->Add( m_radioBtn1, 0, wxALL, 5 );

    m_radioBtn2 = new wxRadioButton( this, wxID_ANY, wxT("Simon-Board"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer18->Add( m_radioBtn2, 0, wxALL, 5 );

	bSizer11->Add( bSizer18, 0, wxEXPAND, 5 );

	bSizer10->Add( bSizer11, 1, wxEXPAND, 5 );

	bSizer1->Add( bSizer10, 1, wxEXPAND, 5 );

	fgSizer1->Add( bSizer1, 1, wxEXPAND, 5 );

	#ifdef __WINDOWS__
	m_textCtrl3 = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize( 490,190 ), wxTE_MULTILINE|wxTE_READONLY );
	#endif

	#ifdef __LINUX__
	m_textCtrl3 = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize( 590,220 ), wxTE_MULTILINE|wxTE_READONLY );
	#endif

	fgSizer1->Add( m_textCtrl3, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5 );

	wxBoxSizer* bSizer17;
	bSizer17 = new wxBoxSizer( wxHORIZONTAL );

	#ifdef __WINDOWS__
	bSizer17->SetMinSize( wxSize( 480,-1 ) );
	#endif

	#ifdef __LINUX__
	bSizer17->SetMinSize( wxSize( 600,-1 ) );
	#endif

	m_button5 = new wxButton( this, ID_TETRIS, wxT("Tetris"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer17->Add( m_button5, 1, wxALL|wxEXPAND, 5 );

	m_button6 = new wxButton( this, ID_PACMAN, wxT("Pacman"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer17->Add( m_button6, 1, wxALL|wxEXPAND, 5 );

	m_button7 = new wxButton( this, ID_MARIO, wxT("Mario"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer17->Add( m_button7, 1, wxALL|wxEXPAND, 5 );

	m_button8 = new wxButton( this, ID_PONG, wxT("Pong"), wxDefaultPosition, wxDefaultSize, 0 );
	bSizer17->Add( m_button8, 1, wxALL|wxEXPAND, 5 );

	fgSizer1->Add( bSizer17, 0, wxALIGN_CENTER_HORIZONTAL|wxEXPAND, 5 );

	this->SetSizer( fgSizer1 );
	this->Layout();

  Connect(ID_HELP, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(RightPanel::OnHelpClick));
  Connect(ID_OPEN, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(RightPanel::OnOpenHexClick));
  Connect(ID_FIND, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnFindDeviceClick));
  Connect(ID_FLASH, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnFlashClick));
  Connect(ID_TETRIS, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnTetrisClick));
  Connect(ID_PACMAN, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnPacmanClick));
  Connect(ID_MARIO, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnMarioClick));
  Connect(ID_PONG, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(CBootloader::OnPongClick));
}


void RightPanel::OnHelpClick(wxCommandEvent& event)
  {
  AppHelp(this);
  }

// Micronucleus Funktionen

micronucleus* CBootloader::micronucleus_connect(Communicate* comm) {
  micronucleus *nucleus = NULL;
  struct usb_bus *busses;

  // intialise usb and find micronucleus device
  usb_init();
  usb_find_busses();
  usb_find_devices();

  busses = usb_get_busses();
  struct usb_bus *bus;
  for (bus = busses; bus; bus = bus->next) {
    struct usb_device *dev;

    for (dev = bus->devices; dev; dev = dev->next) {
      /* Check if this device is a micronucleus */
      if (dev->descriptor.idVendor == MICRONUCLEUS_VENDOR_ID && dev->descriptor.idProduct == MICRONUCLEUS_PRODUCT_ID)  {
        nucleus = (micronucleus*) malloc(sizeof(micronucleus));
        nucleus->version.major = (dev->descriptor.bcdDevice >> 8) & 0xFF;
        nucleus->version.minor = dev->descriptor.bcdDevice & 0xFF;

        if (nucleus->version.major > MICRONUCLEUS_MAX_MAJOR_VERSION) {
          /*
          fprintf(stderr,
                  "Warning: device with unknown new version of Micronucleus detected.\n"
                  "This tool doesn't know how to upload to this new device. Updates may be available.\n"
                  "Device reports version as: %d.%d\n",
                  nucleus->version.major, nucleus->version.minor);
          */

          comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("Warning: device with unknown new version of Micronucleus detected.\n"
                  "This tool doesn't know how to upload to this new device. Updates may be available.\n"
                  "Device reports version as: %d.%d\n"), nucleus->version.major, nucleus->version.minor));

          return NULL;
        }

        nucleus->device = usb_open(dev);

        // get nucleus info
        unsigned char buffer[4];
        int res = usb_control_msg(nucleus->device, 0xC0, 0, 0, 0, (char *)buffer, 4, MICRONUCLEUS_USB_TIMEOUT);
        assert(res >= 4);

        nucleus->flash_size = (buffer[0]<<8) + buffer[1];
        nucleus->page_size = buffer[2];
        nucleus->pages = (nucleus->flash_size / nucleus->page_size);
        if (nucleus->pages * nucleus->page_size < nucleus->flash_size) nucleus->pages += 1;
        nucleus->write_sleep = buffer[3];
        nucleus->erase_sleep = nucleus->write_sleep * nucleus->pages;
      }
    }
  }

  return nucleus;
}

/* Delay in miliseconds */
void delay(unsigned int duration) {
  #if defined _WIN32 || defined _WIN64
    // use windows sleep api with milliseconds
    // * 2 to make it run at half speed, because windows seems to have some trouble with this...
    Sleep(duration * 2);
  #else
    // use standard unix api with microseconds
    usleep(duration*1000);
  #endif
}


int micronucleus_eraseFlash(micronucleus* deviceHandle, micronucleus_callback progress, Communicate* comm) {
  int res;
  res = usb_control_msg(deviceHandle->device, 0xC0, 2, 0, 0, NULL, 0, MICRONUCLEUS_USB_TIMEOUT);

  // give microcontroller enough time to erase all writable pages and come back online
  float i = 0;
  while (i < 1.0) {
    // update progress callback if one was supplied
    if (progress) progress(i, comm);

    delay(((float) deviceHandle->erase_sleep) / 100.0f);
    i += 0.01;
  }

  /* Under Linux, the erase process is often aborted with errors such as:
   usbfs: USBDEVFS_CONTROL failed cmd micronucleus rqt 192 rq 2 len 0 ret -84
   This seems to be because the erase is taking long enough that the device
   is disconnecting and reconnecting.  Under Windows, micronucleus can see this
   and automatically reconnects prior to uploading the program.  To get the
   the same functionality, we must flag this state (the "-84" error result) by
   converting the return to -2 for the upper layer.

   On Mac OS a common error is -34 = epipe, but adding it to this list causes:
   Assertion failed: (res >= 4), function micronucleus_connect, file library/micronucleus_lib.c, line 63.
  */
  if (res == -5 || res == -34 || res == -84) {
    if (res == -34) {
      usb_close(deviceHandle->device);
      deviceHandle->device = NULL;
    }

    return 1; // recoverable errors
  } else {
    return res;
  }
}


int micronucleus_writeFlash(micronucleus* deviceHandle, unsigned int program_size, unsigned char* program, micronucleus_callback prog, Communicate *comm) {

  unsigned char page_length = deviceHandle->page_size;
  unsigned char page_buffer[page_length];
  unsigned int  address; // overall flash memory address
  unsigned int  page_address; // address within this page when copying buffer
  unsigned int  res;
  unsigned int  pagecontainsdata;

  for (address = 0; address < deviceHandle->flash_size; address += deviceHandle->page_size) {
    // work around a bug in older bootloader versions
    if (deviceHandle->version.major == 1 && deviceHandle->version.minor <= 2
        && address / deviceHandle->page_size == deviceHandle->pages - 1) {
      page_length = deviceHandle->flash_size % deviceHandle->page_size;
    }

    pagecontainsdata=0;

    // copy in bytes from user program
    for (page_address = 0; page_address < page_length; page_address += 1) {
      if (address + page_address > program_size) {
        page_buffer[page_address] = 0xFF; // pad out remainder with unprogrammed bytes
      } else {
        pagecontainsdata=1; // page contains data and needs to be written
        page_buffer[page_address] = program[address + page_address]; // load from user program
      }
    }

    // always write last page so bootloader can insert the tiny vector table
    if ( address >= deviceHandle->flash_size - deviceHandle->page_size )
      pagecontainsdata = 1;

    // ask microcontroller to write this page's data
    if (pagecontainsdata) {
      res = usb_control_msg(deviceHandle->device,
             USB_ENDPOINT_OUT| USB_TYPE_VENDOR | USB_RECIP_DEVICE,
             1,
             page_length, address,
             (char*)page_buffer, page_length,
             MICRONUCLEUS_USB_TIMEOUT);

      if (res != page_length) return -1;

      // give microcontroller enough time to write this page and come back online
      delay(deviceHandle->write_sleep);
    }

  // call progress update callback if that's a thing
  if (prog) prog(((float) address) / ((float) deviceHandle->flash_size), comm);
  }

  // call progress update callback with completion status
  if (prog) prog(1.0, comm);

  return 0;
}


int micronucleus_startApp(micronucleus* deviceHandle) {
  int res;
  res = usb_control_msg(deviceHandle->device, 0xC0, 4, 0, 0, NULL, 0, MICRONUCLEUS_USB_TIMEOUT);

  if(res!=0)
    return -1;
  else
    return 0;
}


int CBootloader::parseRaw(char *filename, unsigned char *data_buffer, int *start_address, int *end_address) {
  FILE *input;

  Communicate *comm = (Communicate *) m_parent->GetParent();

  input = strcmp(filename, "-") == 0 ? stdin : fopen(filename, "r");

  if (input == NULL) {
    //printf("> Error reading %s: %s\n", filename, strerror(errno));
    comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Error reading %s: %s\n"), filename, strerror(errno)));
    return 1;
  }

  *start_address = 0;
  *end_address = 0;

  // read in bytes from file
  int byte = 0;
  while (1) {
    byte = getc(input);
    if (byte == EOF) break;

    *data_buffer = byte;
    data_buffer += 1;
    *end_address += 1;
  }

  fclose(input);
  return 0;
}


int CBootloader::parseHex(FILE *file_pointer, int num_digits) {
  int iter;
  char temp[9];

  for(iter = 0; iter < num_digits; iter++) {
    temp[iter] = getc(file_pointer);
  }
  temp[iter] = 0;

  return strtol(temp, NULL, 16);
}


int CBootloader::parseIntelHex(char *hexfile, unsigned char *buffer, int *startAddr, int *endAddr) {
  int address, base, d, segment, i, lineLen, sum;
  FILE *input;

  Communicate *comm = (Communicate *) m_parent->GetParent();

  input = strcmp(hexfile, "-") == 0 ? stdin : fopen(hexfile, "r");
  if (input == NULL) {
    //printf("> Error opening %s: %s\n", hexfile, strerror(errno));
    comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Error opening %s: %s\n"), hexfile, strerror(errno)));
    return 1;
  }

  while (parseUntilColon(input) == ':') {
    sum = 0;
    sum += lineLen = parseHex(input, 2);
    base = address = parseHex(input, 4);
    sum += address >> 8;
    sum += address;
    sum += segment = parseHex(input, 2);  /* segment value? */
    if (segment != 0) {   /* ignore lines where this byte is not 0 */
      continue;
    }

    for (i = 0; i < lineLen; i++) {
      d = parseHex(input, 2);
      buffer[address++] = d;
      sum += d;
    }

    sum += parseHex(input, 2);
    if ((sum & 0xff) != 0) {
      //printf("> Warning: Checksum error between address 0x%x and 0x%x\n", base, address);
      comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Warning: Checksum error between address 0x%x and 0x%x\n"), base, address));
    }

    if(*startAddr > base) {
      *startAddr = base;
    }
    if(*endAddr < address) {
      *endAddr = address;
    }
  }

  fclose(input);
  return 0;
}


int CBootloader::parseUntilColon(FILE *file_pointer) {
  int character;

  do {
    character = getc(file_pointer);
  } while(character != ':' && character != EOF);

  return character;
}


void printProgress(float progress, Communicate* comm) {
  static int last_step;
  static int last_integer_total_progress;

  if (dump_progress) {

    if ((progress_step==1) && (progress==0.5))
      {
      //clear Textctrl
      comm->m_rp->m_textCtrl3->SetInsertionPoint(0);
      comm->m_rp->m_textCtrl3->SetValue("\n\n\n\n\n\n\n\n\n\n\n");
      comm->m_rp->m_textCtrl3->WriteText("           \n               \n                ");
      comm->m_rp->m_textCtrl3->SetInsertionPoint(0);
      comm->m_rp->m_textCtrl3->SetValue("");
      }

    comm->m_rp->m_textCtrl3->WriteText(wxString::Format(wxT("{status:\"%s\",step:%d,steps:%d,progress:%f}\n"), progress_friendly_name, progress_step, progress_total_steps, progress));

  } else {

    if (last_step == progress_step && use_ansi) comm->m_rp->m_textCtrl3->SetValue(content);

    else
     {
     content = comm->m_rp->m_textCtrl3->GetValue();
     }

    float total_progress = ((float) progress_step - 1.0f) / (float) progress_total_steps;
    total_progress += progress / (float) progress_total_steps;
    int integer_total_progress = total_progress * 100.0f;

    if (use_ansi || integer_total_progress >= last_integer_total_progress + 5) {

    //printf("%s: %d%% complete\n", progress_friendly_name, integer_total_progress);

    comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("%s: %d%% complete\n"), progress_friendly_name, integer_total_progress));

    last_integer_total_progress = integer_total_progress;
    }
  }

  last_step = progress_step;
}


void CBootloader::setProgressData(char* friendly, int step) {
  progress_friendly_name = friendly;
  progress_step = step;
}


int CBootloader::MicronucleusWriteFlash()
{
  Communicate *comm = (Communicate *) m_parent->GetParent();

  int res;
  // parse arguments
  int file_type = FILE_TYPE_INTEL_HEX;
  progress_step = 0;
  if(comm->m_rp->m_checkBox1->GetValue()) progress_total_steps = 6;
  else progress_total_steps = 5; // steps: waiting, connecting, parsing, erasing, writing, (running)?
  dump_progress = 0;
  erase_only = 0;
  timeout = 5;
  use_ansi = 1;

  if ((comm->m_rp->filename) != "")
    {
    comm->m_rp->m_staticText9->SetLabel(comm->m_rp->filename);
    comm->m_rp->m_textCtrl3->SetValue("Die Hex-Datei " + comm->m_rp->filename
                                    + " wurde ausgewhlt.\n\n");
    }
  else
    {
    comm->m_rp->m_textCtrl3->SetValue("Es wurde keine Hex-Datei ausgewhlt !\n");
    return EXIT_FAILURE;
    }


  char *file = (char*)comm->m_rp->path.mb_str(wxConvUTF8);
  content = comm->m_rp->m_textCtrl3->GetValue();

  comm->m_rp->m_button1->Enable(false);
  comm->m_rp->m_button3->Enable(false);

  setProgressData("waiting", 1);

  if (dump_progress) printProgress(0.5, comm);
  //printf("> Press CTRL+C to terminate the program.\n");
  //comm->m_rp->m_textCtrl3->AppendText("> Press CTRL+C to terminate the program.\n");

  else {
  //clear Textctrl
  comm->m_rp->m_textCtrl3->SetInsertionPoint(0);
  comm->m_rp->m_textCtrl3->SetValue("\n\n\n\n\n\n\n\n\n\n\n");
  comm->m_rp->m_textCtrl3->WriteText("           \n               \n                ");
  comm->m_rp->m_textCtrl3->SetInsertionPoint(0);
  comm->m_rp->m_textCtrl3->SetValue("");}

  comm->m_rp->m_textCtrl3->WriteText("> Please plug in the device ... \n");

  comm->m_rp->m_textCtrl1->SetInsertionPoint(0);
  comm->m_rp->m_textCtrl2->SetInsertionPoint(0);
  comm->m_rp->m_textCtrl1->SetValue("");
  comm->m_rp->m_textCtrl2->SetValue("");
  comm->m_rp->m_textCtrl1->WriteText("16d0");
  comm->m_rp->m_textCtrl2->WriteText("753");

  time_t start_time, current_time;
  time(&start_time);

  micronucleus *my_device = NULL;

  while (my_device == NULL) {
    delay(100);
    my_device = micronucleus_connect(comm);

    time(&current_time);
    if (timeout && start_time + timeout < current_time) {
      comm->m_rp->m_textCtrl3->WriteText("> Timeout !\n");
      break;
    }
  }

  if (my_device == NULL) {
    //printf("> Device search timed out\n");
    comm->m_rp->m_textCtrl3->WriteText("> Device search timed out\n");
    comm->m_rp->m_button1->Enable(true);
    comm->m_rp->m_button3->Enable(true);

    return EXIT_FAILURE;
  }

  //printf("> Device is found!\n");
  comm->m_rp->m_textCtrl3->AppendText("> Device is found!\n");

  // wait for CONNECT_WAIT milliseconds with progress output
  float wait = 0.0f;
  setProgressData("connecting", 2);
  while (wait < CONNECT_WAIT) {
    printProgress((wait / ((float) CONNECT_WAIT)) * 0.9f, comm);
    wait += 50.0f;
    delay(50);
  }

  printProgress(1.0, comm);

  // if (my_device->page_size == 64) {
  //   printf("> Device looks like ATtiny85!\n");
  // } else if (my_device->page_size == 32)  {
  //   printf("> Device looks like ATtiny45!\n");
  // } else {
  //   printf("> Unsupported device!\n");
  //   return EXIT_FAILURE;
  // }

  //printf("> Device has firmware version %d.%d\n",my_device->version.major,my_device->version.minor);
  comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Device has firmware version %d.%d\n"), my_device->version.major, my_device->version.minor));
  //printf("> Available space for user applications: %d bytes\n", my_device->flash_size);
  comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Available space for user applications: %d bytes\n"), my_device->flash_size));
  //printf("> Suggested sleep time between sending pages: %ums\n", my_device->write_sleep);
  comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Suggested sleep time between sending pages: %ums\n"), my_device->write_sleep));
  //printf("> Whole page count: %d  page size: %d\n", my_device->pages,my_device->page_size);
  comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Whole page count: %d  page size: %d\n"), my_device->pages, my_device->page_size));
  //printf("> Erase function sleep duration: %dms\n", my_device->erase_sleep);
  comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Erase function sleep duration: %dms\n"), my_device->erase_sleep));

  //show firmware
  comm->m_rp->m_staticText3->SetLabel("micronucleus");

  int startAddress = 1, endAddress = 0;


  if (!erase_only) {
    setProgressData("parsing", 3);
    printProgress(0.0, comm);
    memset(dataBuffer, 0xFF, sizeof(dataBuffer));


    if (file_type == FILE_TYPE_INTEL_HEX) {

      if (parseIntelHex(file, dataBuffer, &startAddress, &endAddress)) {
        //printf("> Error loading or parsing hex file.\n");
        comm->m_rp->m_textCtrl3->AppendText("> Error loading or parsing hex file.\n");

        comm->m_rp->m_button1->Enable(true);
        comm->m_rp->m_button3->Enable(true);

        return EXIT_FAILURE;
      }
    } else if (file_type == FILE_TYPE_RAW) {
      if (parseRaw(file, dataBuffer, &startAddress, &endAddress)) {
        //printf("> Error loading raw file.\n");
        comm->m_rp->m_textCtrl3->AppendText("> Error loading raw file.\n");

        comm->m_rp->m_button1->Enable(true);
        comm->m_rp->m_button3->Enable(true);

        return EXIT_FAILURE;
      }
    }

    printProgress(1.0, comm);

    if (startAddress >= endAddress) {
      //printf("> No data in input file, exiting.\n");
      comm->m_rp->m_textCtrl3->AppendText("> No data in input file, exiting.\n");

      comm->m_rp->m_button1->Enable(true);
      comm->m_rp->m_button3->Enable(true);

      return EXIT_FAILURE;
    }

    // pagesize
    comm->m_rp->m_staticText6->SetLabel(wxString::Format(wxT("%.8d"), my_device->page_size));
    // programsize
    comm->m_rp->m_staticText8->SetLabel(wxString::Format(wxT("%.8d"), endAddress));

    if (endAddress > my_device->flash_size) {
      //printf("> Program file is %d bytes too big for the bootloader!\n", endAddress - my_device->flash_size);
      comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT("> Program file is %d bytes too big for the bootloader!\n"), endAddress - my_device->flash_size));

      comm->m_rp->m_button1->Enable(true);
      comm->m_rp->m_button3->Enable(true);

      return EXIT_FAILURE;
    }

  }

  printProgress(1.0, comm);

  setProgressData("erasing", 4);
  //printf("> Erasing the memory ...\n");
  comm->m_rp->m_textCtrl3->AppendText("> Erasing the memory ...\n");

  res = micronucleus_eraseFlash(my_device, printProgress, comm);

  if (res == 1) { // erase disconnection bug workaround
    //printf(">> Eep! Connection to device lost during erase! Not to worry\n");
    comm->m_rp->m_textCtrl3->AppendText(">> Eep! Connection to device lost during erase! Not to worry\n");
    //printf(">> This happens on some computers - reconnecting...\n");
    comm->m_rp->m_textCtrl3->AppendText(">> This happens on some computers - reconnecting...\n");

    my_device = NULL;

    delay(CONNECT_WAIT);

    int deciseconds_till_reconnect_notice = 50; // notice after 5 seconds
    while (my_device == NULL) {
      delay(100);
      my_device = micronucleus_connect(comm);
      deciseconds_till_reconnect_notice -= 1;

      if (deciseconds_till_reconnect_notice == 0) {
        //printf(">> (!) Automatic reconnection not working. Unplug and reconnect\n");
        comm->m_rp->m_textCtrl3->AppendText(">> (!) Automatic reconnection not working. Unplug and reconnect\n");
        //printf("   device usb connector, or reset it some other way to continue.\n");
        comm->m_rp->m_textCtrl3->AppendText("   device usb connector, or reset it some other way to continue.\n");
      }
    }

    //printf(">> Reconnected! Continuing upload sequence...\n");
    comm->m_rp->m_textCtrl3->AppendText(">> Reconnected! Continuing upload sequence...\n");

  } else if (res != 0) {
    //printf(">> Flash erase error %d has occured ...\n", res);
    comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT(">> Flash erase error %d has occured ...\n"), res));

    //printf(">> Please unplug the device and restart the program.\n");
    comm->m_rp->m_textCtrl3->AppendText(">> Please unplug the device and restart the program.\n");

    comm->m_rp->m_button1->Enable(true);
    comm->m_rp->m_button3->Enable(true);

    return EXIT_FAILURE;
  }
  printProgress(1.0, comm);

  if (!erase_only) {
    //printf("> Starting to upload ...\n");
    comm->m_rp->m_textCtrl3->AppendText("> Starting to upload ...\n");
    setProgressData("writing", 5);
    res = micronucleus_writeFlash(my_device, endAddress, dataBuffer, printProgress, comm);
    if (res != 0) {
      //printf(">> Flash write error %d has occured ...\n", res);
      comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT(">> Flash write error %d has occured ...\n"), res));
      //printf(">> Please unplug the device and restart the program.\n");
      comm->m_rp->m_textCtrl3->AppendText(">> Please unplug the device and restart the program.\n");

      comm->m_rp->m_button1->Enable(true);
      comm->m_rp->m_button3->Enable(true);

      return EXIT_FAILURE;
    }
  }

  if (comm->m_rp->m_checkBox1->GetValue()) {
    //printf("> Starting the user app ...\n");
    comm->m_rp->m_textCtrl3->AppendText("> Starting the user app ...\n");
    setProgressData("running", 6);
    printProgress(1.0, comm);

    res = micronucleus_startApp(my_device);

    if (res != 0) {
      //printf(">> Run error %d has occured ...\n", res);
      comm->m_rp->m_textCtrl3->AppendText(wxString::Format(wxT(">> Run error %d has occured ...\n"), res));
      //printf(">> Please unplug the device and restart the program. \n");
      comm->m_rp->m_textCtrl3->AppendText(">> Please unplug the device and restart the program.\n");

      comm->m_rp->m_button1->Enable(true);
      comm->m_rp->m_button3->Enable(true);

      return EXIT_FAILURE;
    }

    printProgress(1.0, comm);
  }

  //printf(">> Micronucleus done. Thank you!\n");
  comm->m_rp->m_textCtrl3->AppendText(">> Micronucleus done. Thank you!\n");

  comm->m_rp->m_button1->Enable(true);
  comm->m_rp->m_button3->Enable(true);

  return EXIT_SUCCESS;
}

void CBootloader::WriteFlash()
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();
  comm->m_rp->m_staticText3->SetLabel("                     ");
  comm->m_rp->m_staticText6->SetLabel("00000000");
  comm->m_rp->m_staticText8->SetLabel("00000000");
  comm->m_rp->m_textCtrl3->SetValue("");
  comm->m_rp->m_textCtrl1->SetValue("16c0");
  comm->m_rp->m_textCtrl2->SetValue("5dc");

  if ((comm->m_rp->filename) != "")
  {
    Communicate *comm = (Communicate *) m_parent->GetParent();

    CBootloader * bootloader = new CBootloader(comm);

    if (bootloader->error == 0)
      {
      unsigned int pagesize = bootloader->getPagesize(comm);

      comm->m_rp->m_staticText6->SetLabel(wxString::Format(wxT("%.8d"),(int)pagesize));

      CFlashmem * flashmem = new CFlashmem(pagesize, comm);

      if ((comm->m_rp->filename) != "")
        {
        comm->m_rp->m_staticText9->SetLabel(comm->m_rp->filename);
        comm->m_rp->m_textCtrl3->AppendText("Die Hex-Datei " + comm->m_rp->filename
                                          + " wurde ausgewhlt.\n\n");

        flashmem->readFromIHEX((char*)comm->m_rp->path.mb_str(wxConvUTF8), comm);
        }
      else
        {
        comm->m_rp->m_textCtrl3->AppendText("Es wurde keine Hex-Datei ausgewhlt !\n");
        error = 1;
        }

      if (flashmem->error == 0)
        {
        CPage* pPage = flashmem->getFirstpage();

        while (pPage != NULL) {

        comm->m_rp->m_textCtrl3->AppendText("Schreibe Daten an Adresse: "
        + wxString::Format(wxT("%d"),(int)pPage->getPageaddress())+"\n");

        bootloader->writePage(pPage, comm);
        pPage = pPage->getNext();
        }

        unsigned int devicesize = (flashmem->m_nPagecount) * pagesize;
        comm->m_rp->m_staticText8->SetLabel(wxString::Format(wxT("%.8d"),(int)devicesize));

        comm->m_rp->m_textCtrl3->AppendText("\nDie Hex-Datei " + comm->m_rp->m_staticText9->GetLabelText()
                                          + " wurde in den Speicher bertragen.\n");

        if (comm->m_rp->m_checkBox1->GetValue()) bootloader->startApplication(comm);
        }
      }
    }
  else
    {
    comm->m_rp->m_textCtrl3->AppendText("Es wurde keine Hex-Datei ausgewhlt !\n");
    }
  }


void RightPanel::OnOpenHexClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();

  wxFileDialog openFileDialog(this, _("Hex-Datei auswhlen"), "", "", "Hex-Dateien (*.hex)|*.hex", wxFD_OPEN|wxFD_FILE_MUST_EXIST);

  if (openFileDialog.ShowModal() == wxID_OK)
    {
    filename = openFileDialog.GetFilename();
    path = openFileDialog.GetPath();
    comm->m_rp->m_staticText9->SetLabel(filename);
    }
  }


void CBootloader::OnFindDeviceClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();
  comm->m_rp->m_staticText3->SetLabel("                     ");
  comm->m_rp->m_staticText6->SetLabel("00000000");
  comm->m_rp->m_staticText8->SetLabel("00000000");
  comm->m_rp->m_textCtrl3->SetValue("");

  if ((comm->m_rp->m_textCtrl1->GetValue()=="") || (comm->m_rp->m_textCtrl2->GetValue()==""))
    {
    comm->m_rp->m_textCtrl3->AppendText("Warnung: Es wurde keine Vendor- bzw. Produkt-ID eingegeben !\n\n");
    }
  else CBootloader * bootloader = new CBootloader(comm);
  }


void CBootloader::OnFlashClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();
  comm->m_rp->m_staticText3->SetLabel("                     ");
  comm->m_rp->m_staticText6->SetLabel("00000000");
  comm->m_rp->m_staticText8->SetLabel("00000000");

  if (comm->m_rp->m_radioBtn1->GetValue()) WriteFlash();
  if (comm->m_rp->m_radioBtn2->GetValue()) MicronucleusWriteFlash();

  }


void CBootloader::OnTetrisClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();
  comm->m_rp->m_textCtrl1->SetValue("16c0");
  comm->m_rp->m_textCtrl2->SetValue("5dc");
  comm->m_rp->m_staticText9->SetLabel("tetris.hex");
  comm->m_rp->filename=wxT("tetris.hex");

  wxString ExecutableDirectory = wxPathOnly(wxStandardPaths::Get().GetExecutablePath());

  #ifdef __WINDOWS__
  if(!wxFileExists(ExecutableDirectory + wxT("\\tetris.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("\\tetris.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("\\tetris.hex");
    WriteFlash();
    }
  #endif

  #ifdef __LINUX__
  if(!wxFileExists(ExecutableDirectory + wxT("/tetris.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("/tetris.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("/tetris.hex");
    WriteFlash();
    }
  #endif
  }


void CBootloader::OnPacmanClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();
  comm->m_rp->m_textCtrl1->SetValue("16c0");
  comm->m_rp->m_textCtrl2->SetValue("5dc");
  comm->m_rp->m_staticText9->SetLabel("pacman.hex");
  comm->m_rp->filename=wxT("pacman.hex");

  wxString ExecutableDirectory = wxPathOnly(wxStandardPaths::Get().GetExecutablePath());

  #ifdef __WINDOWS__
  if(!wxFileExists(ExecutableDirectory + wxT("\\pacman.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("\\pacman.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("\\pacman.hex");
    WriteFlash();
    }
  #endif

  #ifdef __LINUX__
  if(!wxFileExists(ExecutableDirectory + wxT("/pacman.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("/pacman.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("/pacman.hex");
    WriteFlash();
    }
  #endif
  }


void CBootloader::OnMarioClick(wxCommandEvent& event)
  {
  Communicate *comm = (Communicate *) m_parent->GetParent();

  comm->m_rp->m_textCtrl1->SetValue("16c0");
  comm->m_rp->m_textCtrl2->SetValue("5dc");

  comm->m_rp->m_staticText9->SetLabel("mario.hex");
  comm->m_rp->filename=wxT("mario.hex");

  wxString ExecutableDirectory = wxPathOnly(wxStandardPaths::Get().GetExecutablePath());

  #ifdef __WINDOWS__
  if(!wxFileExists(ExecutableDirectory + wxT("\\mario.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("\\mario.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("\\mario.hex");
    WriteFlash();
    }
  #endif

  #ifdef __LINUX__
  if(!wxFileExists(ExecutableDirectory + wxT("/mario.hex")))
    {
    wxMessageBox(ExecutableDirectory + wxT("/mario.hex wurde nicht gefunden !"), wxT("Info"), wxICON_INFORMATION);
    }
  else
    {
    comm->m_rp->path=ExecutableDirectory+wxT("/mario.hex");
    WriteFlash();
    }
  #endif
  }


void CBootloader::OnPongClick(wxCommandEvent& event)
  {
  wxMessageBox( wxT("Diese Funktion ist noch nicht implementiert !"), wxT("Info"), wxICON_INFORMATION);
  }


